// aptcp.cpp
// ---------------------------------------------------------------------------------------------------
#include <sstream>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <set>
#include <iterator>
#include <utility>
#include <cstring>
#include <ctime>
#include <xalanc/Include/PlatformDefinitions.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xalanc/XalanTransformer/XalanTransformer.hpp>
#include "aptcp.hpp"
#include "aptcp_college.hpp"
#include "aptcp_fetch_page.hpp"
#include "aptcp_tidydoc.hpp"

int main() {
   // ----- Create XML document content by concatenating the table from each college's policy page.

   std::stringstream xml_doc(APTCP_XML_DOC_PREFIX, std::ios::in | std::ios::out | std::ios::ate);
   {
      std::set<College> colleges {}; std::copy(std::istream_iterator<College> {std::cin}, std::istream_iterator<College> {},
         std::inserter(colleges, colleges.begin()));
      for (auto c : colleges) {
         auto page {tidy_up_html(fetch_page(c.url()))};
         if (auto b {page.find(APTCP_CONTENT_BEGIN)}; b != std::string::npos) {	                                     // Has content?
            xml_doc << page.substr(b).substr(0,page.substr(b).find(APTCP_CONTENT_END)+std::strlen(APTCP_CONTENT_END)) // Extract it.
               << "<info>" << c << "</info>\n";
            }
         else {
            xml_doc << "<warning>" << c << "</warning>\n";
            }
         }
      }
   xml_doc << APTCP_XML_DOC_POSTFIX;

   // ----- Create XSLT style sheet.

   std::stringstream style_sheet(APTCP_XSLT, std::ios::in | std::ios::out | std::ios::ate);

	// ----- Create the transformer.

   XALAN_USING_XERCES(XMLPlatformUtils)
   XALAN_USING_XALAN(XalanTransformer)
   XMLPlatformUtils::Initialize();
   XalanTransformer::initialize();
   XalanTransformer transformer {};
   {
      auto now {std::time(nullptr)};
      transformer.setStylesheetParam("current_time_p", ("'" + std::string {std::asctime(std::localtime(&now))}+ "'").c_str());
      }

// ----- Use the transformer to invoke the XSLT style sheet on the XML document.

   if (transformer.transform(xml_doc, style_sheet, std::cout) != 0)
      std::cerr << "aptcp/main()/transformer.getLastError(): " << transformer.getLastError() << "\n" << style_sheet.str() << xml_doc.str();
   }
