#ifndef APTCP_HPP_
#define APTCP_HPP_
// aptcp.hpp
// ---------------------------------------------------------------------------------------------------
#include <string>

constexpr auto APTCP_CREDIT_POLICY_HOST        {"apstudent.collegeboard.org"};
constexpr auto APTCP_CREDIT_POLICY_TARGET_BASE {"/creditandplacement/credit-policy-detail?diCode="};
constexpr auto APTCP_CONTENT_BEGIN             {"<table id=\"creditPolicyTable\" "};
constexpr auto APTCP_CONTENT_END               {"</table>"};
constexpr auto APTCP_XML_DOC_PREFIX            {R"EOD(<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE doc [<!ENTITY nbsp "&#xa0;">]>
<doc>
<head>
</head>
<body>
)EOD"};
constexpr auto APTCP_XML_DOC_POSTFIX           {R"EOD(</body></doc>)EOD"};
constexpr auto APTCP_XSLT {R"EOD(<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet id="aptcp_stylesheet"
                version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output
            method="html"
            version="4.01"
            indent="yes"
            doctype-system="http://www.w3.org/TR/html4/strict.dtd"
            doctype-public="-//W3C//DTD HTML 4.01//EN"/>
<xsl:template match="xsl:stylesheet" />
<xsl:param name="current_time_p"/>
<xsl:template match="/">
  <html>
  <body>
    <style>
    pp {display: none;}
    pp, table, tr {
        width: 100%;
    }
    th, td {
       border: 1px solid black;
       width: 20%;
    }
    </style>
    <h2>College Entrance Exam Board<br />Advance Placement (AP) Test Credit Policies<br />for Selected Colleges</h2>
    <p>Generated by APTCP Rel 2.5-2(brnNN-1804dd) at: <xsl:value-of select="$current_time_p"/></p>
    <h3>Select Tests</h3>
    <xsl:for-each select="doc/body/table/tbody/tr/td[1][not(.=preceding::*)]">
        <xsl:sort select="."/>
        <xsl:variable name="testString" select="."/>
        <xsl:variable name="testName" select="translate(normalize-space($testString),' :-','')"/>
        <xsl:variable name="testClass" select="concat('class-',$testName)"/>
        <xsl:variable name="testCheckbox" select="concat('checkbox-',$testName)"/>
        <style>
            .<xsl:value-of select="$testClass"/> {display: none;}
            #<xsl:value-of select="$testCheckbox"/>:checked ~ .<xsl:value-of select="$testClass"/> {display: table;}
        </style>
        |
        <input>
            <xsl:attribute name="id"><xsl:value-of select="$testCheckbox"/></xsl:attribute>
            <xsl:attribute name="type">checkbox</xsl:attribute>
            </input>
        <xsl:value-of select="$testString"/>
    </xsl:for-each>
    
    <h3>Policies</h3>
    
    <table><tr><th>College</th><th>Test</th><th>Min. score</th><th>Credits</th><th>Course</th></tr></table>
    <xsl:for-each select="doc/body/table/tbody/tr">
        <xsl:variable name="collegeString" select="substring-before(../../caption,'Credit')"/>
        <xsl:variable name="testString">
            <xsl:choose><xsl:when test="td[1] != ''">
                <xsl:value-of select="td[1]"/>
            </xsl:when><xsl:when test="preceding-sibling::*[1]/td[1] != ''">
                <xsl:value-of select="preceding-sibling::*[1]/td[1]" />
            </xsl:when><xsl:when test="preceding-sibling::*[2]/td[1] != ''">
                <xsl:value-of select="preceding-sibling::*[2]/td[1]" />
            </xsl:when><xsl:when test="preceding-sibling::*[3]/td[1] != ''">
                <xsl:value-of select="preceding-sibling::*[3]/td[1]" />
            </xsl:when><xsl:when test="preceding-sibling::*[4]/td[1] != ''">
                <xsl:value-of select="preceding-sibling::*[4]/td[1]" />
            </xsl:when><xsl:otherwise>
                <xsl:value-of select="preceding-sibling::*[5]/td[1]" />
            </xsl:otherwise></xsl:choose>
        </xsl:variable>
        <xsl:variable name="testName" select="translate(normalize-space($testString),' :-','')"/>
        <xsl:variable name="testClass" select="concat('class-',$testName)"/>
        <xsl:variable name="scoreString" select="td[2]"/>
        <xsl:variable name="creditsString" select="td[3]"/>
        <xsl:variable name="unitsString" select="td[4]"/>
            <pp><xsl:attribute name="class"><xsl:value-of select="$testClass"/></xsl:attribute>
            <table><tr>
            <td><xsl:value-of select="$collegeString"/></td>
            <td><xsl:value-of select="$testString"/></td>
            <td><xsl:value-of select="$scoreString"/></td>
            <td><xsl:value-of select="$creditsString"/></td>
            <td><xsl:value-of select="$unitsString"/></td>
            </tr></table></pp>
    </xsl:for-each>
   
   <h3>Info</h3>

   <p>This report include these colleges. Follow link for details.</p>
   <ol>
   <xsl:for-each select="doc/body/info">
        <li><a><xsl:attribute name="href">http://apstudent.collegeboard.org/creditandplacement/credit-policy-detail?diCode=<xsl:value-of select="substring-before(.,' ')"/></xsl:attribute>
               <xsl:value-of select="."/>
               </a>
            </li>
   </xsl:for-each>
   </ol>
   
   <h3>Warnings</h3>

   <p>These colleges do not have a policies table available for processing by this report. Follow link for details.</p>
   <ol>
   <xsl:for-each select="doc/body/warning">
        <li><a><xsl:attribute name="href">http://apstudent.collegeboard.org/creditandplacement/credit-policy-detail?diCode=<xsl:value-of select="substring-before(.,' ')"/></xsl:attribute>
               <xsl:value-of select="."/>
               </a>
            </li>
   </xsl:for-each>
   </ol>
       
  <p>End of report.</p>
  </body>
  </html>
</xsl:template>
</xsl:stylesheet>
)EOD"};

#endif /* APTCP_HPP_ */
