#ifndef APTCP_FETCH_PAGE_HPP_
#define APTCP_FETCH_PAGE_HPP_
// aptcp_fetch_page.hpp
// ---------------------------------------------------------------------------------------------------
#include "aptcp_root_certificates.hpp"
#include <beast/core.hpp>
#include <beast/http.hpp>
#include <beast/version.hpp>
#include <boost/asio.hpp>
#include <boost/asio/ssl.hpp>
#include <string>
#include <utility>
#include <iostream>

using tcp = boost::asio::ip::tcp; // from <boost/asio.hpp>
namespace ssl = boost::asio::ssl; // from <boost/asio/ssl.hpp>
namespace http = beast::http;     // from <beast/http.hpp>

auto fetch_page(std::pair<std::string,std::string> const url_a){
    auto const [site_a, page_a] {url_a};
    boost::system::error_code bad {};
    auto const die = [&bad,&site_a,&page_a](std::string why) {
       std::cerr << "aptcp_fetch_page(site_a:" + site_a + ",page_a:" + page_a + ")/" + why + ": " + bad.message() << std::endl;
       throw  why + ": " + bad.message();
       return EXIT_FAILURE;
       };
    boost::asio::io_service ios {};
    tcp::resolver r{ios};
    tcp::socket sock{ios};
    std::string const PROTOCOL {"https"};
    auto const lookup {r.resolve({site_a, PROTOCOL}, bad)}; !bad || die("resolve");
    boost::asio::connect(sock, lookup, bad), !bad || die("connect");
    ssl::context ctx{ssl::context::sslv23_client};
       load_root_certificates(ctx, bad), !bad || die("certificate");
    ssl::stream<tcp::socket&> stream{sock, ctx};
       stream.set_verify_mode(ssl::verify_peer | ssl::verify_fail_if_no_peer_cert);
       stream.handshake(ssl::stream_base::client, bad), !bad || die("handshake");
    http::request<http::string_body> req {};
       req.method(http::verb::get);
       req.target(page_a);
       req.version = 11;
       req.set(http::field::host, site_a + ":" + std::to_string(sock.remote_endpoint().port()));
       req.set(http::field::user_agent, BEAST_VERSION_STRING);
       req.prepare_payload();
       http::write(stream, req, bad), !bad || die("write");
    beast::flat_buffer b {};
       http::response<http::dynamic_body> res {};
       http::read(stream, b, res, bad), !bad || die("read");
       std::stringstream page {}; page << res << std::endl;
    if(stream.shutdown(bad); bad && bad != boost::asio::error::eof) die("ssl_shutdown ");
    return page.str();
    }

#endif /* APTCP_FETCH_PAGE_HPP_ */
