#ifndef APTCP_TIDYDOC_HPP_
#define APTCP_TIDYDOC_HPP_
// aptcp_tidydoc.hpp
// ---------------------------------------------------------------------------------------------------
#include <optional>
#include <tidy/tidy.h>
#include <tidy/tidybuffio.h>

class AptcpTidyDoc {
   TidyDoc tdoc_ {tidyCreate()};
   TidyBuffer errbuf_ {0,0,0,0,0};
   TidyBuffer outbuf_ {0,0,0,0,0};
   int rc_ {-1};
public:
   AptcpTidyDoc(){
      tidyOptSetInt (tdoc_, TidyIndentContent, yes);
      tidyOptSetBool(tdoc_, TidyXhtmlOut, yes);
      tidyOptSetBool(tdoc_, TidyForceOutput,yes);
      }
   ~AptcpTidyDoc() {
      tidyBufFree(&outbuf_);
      tidyBufFree(&errbuf_);
      tidyRelease(tdoc_);
      }
   AptcpTidyDoc(AptcpTidyDoc const&)            = delete;   // Rule of 5.
   AptcpTidyDoc(AptcpTidyDoc&&)                 = delete;
   AptcpTidyDoc& operator=(AptcpTidyDoc const&) = delete;
   AptcpTidyDoc& operator=(AptcpTidyDoc&&)      = delete;
   operator TidyDoc() const {return tdoc_;}                 // Convert object back to C for use in C tidy functions
   auto getErr(){return (char*)((errbuf_.bp)?(char*)(errbuf_.bp):"No message.");}
   auto getOut(){return (char*)((outbuf_.bp)?(char*)(outbuf_.bp):"No output.");}
   auto is_ok(){return (rc_>=0);}
   auto is_ok(int code_a){return (rc_=code_a)>=0;}

   AptcpTidyDoc* tidy_up_html_(std::string ugly_html_a){
	   is_ok(tidySetErrorBuffer(tdoc_,&errbuf_))
         && is_ok(tidyParseString(tdoc_,ugly_html_a.c_str()))
         && is_ok(tidyCleanAndRepair(tdoc_))
         && is_ok(tidyRunDiagnostics(tdoc_))
         && is_ok(tidySaveBuffer(tdoc_,&outbuf_));
      return this;
      }
   };

auto tidy_up_html(std::string ugly_html_a) {
   AptcpTidyDoc tdoc{};
   if (!tdoc.tidy_up_html_(ugly_html_a)->is_ok()){
      std::cerr << "\ntidy_file("+ugly_html_a+") " + tdoc.getErr();
      return std::string {};
      }
   return std::string{tdoc.getOut()};
   }

#endif /* APTCP_TIDYDOC_HPP_ */
